@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import math
+import random
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from typing import Any, Optional
@@ -52,6 +53,7 @@ __all__ = [
     'AudioToMelSpectrogramPreprocessor',
     'AudioToMFCCPreprocessor',
     'SpectrogramAugmentation',
+    'MaskedPatchAugmentation',
     'CropOrPadSpectrogramAugmentation',
 ]
 
@@ -516,6 +518,72 @@ class SpectrogramAugmentation(NeuralModule):
         return augmented_spec
 
 
+class MaskedPatchAugmentation(NeuralModule):
+    """
+        Zeroes out fixed size time patches of the spectrogram.
+        All samples in batch are guaranteed to have the same amount of masked time steps.
+        Optionally also performs frequency masking in the same way as SpecAugment.
+        Args:
+            patch_size (int): up to how many time steps does one patch consist of.
+            Defaults to 48.
+            mask_patches (int): how many patches should be masked in each sample.
+            Defaults to 10.
+            freq_masks (int): how many frequency segments should be cut.
+            Defaults to 0.
+            freq_width (int): maximum number of frequencies to be cut in a segment.
+            Defaults to 0.
+    """
+
+    @property
+    def input_types(self):
+        """Returns definitions of module input types
+        """
+        return {
+            "input_spec": NeuralType(('B', 'D', 'T'), SpectrogramType()),
+            "length": NeuralType(tuple('B'), LengthsType()),
+        }
+
+    @property
+    def output_types(self):
+        """Returns definitions of module output types
+        """
+        return {"augmented_spec": NeuralType(('B', 'D', 'T'), SpectrogramType())}
+
+    def __init__(
+        self, patch_size: int = 48, mask_patches: int = 10, freq_masks: int = 0, freq_width: int = 0,
+    ):
+        super().__init__()
+        self.patch_size = patch_size
+        self.mask_patches = mask_patches
+
+        if freq_masks > 0:
+            self.spec_augment = SpecAugment(freq_masks=freq_masks, time_masks=0, freq_width=freq_width, time_width=0,)
+        else:
+            self.spec_augment = None
+
+    @typecheck()
+    def forward(self, input_spec, length):
+        augmented_spec = input_spec
+
+        min_len = torch.min(length)
+        mask_patches = self.mask_patches
+        if min_len < self.patch_size * self.mask_patches:
+            mask_patches = min_len // self.patch_size
+
+        for idx in range(input_spec.shape[0]):
+            cur_len = length[idx]
+            patches = range(cur_len // self.patch_size - 1)
+            masked_patches = random.sample(patches, mask_patches)
+
+            for mp in masked_patches:
+                augmented_spec[idx, :, mp * self.patch_size : (mp + 1) * self.patch_size] = 0.0
+
+        if self.spec_augment is not None:
+            augmented_spec = self.spec_augment(input_spec=augmented_spec, length=length)
+
+        return augmented_spec
+
+
 class CropOrPadSpectrogramAugmentation(NeuralModule):
     """
     Pad or Crop the incoming Spectrogram to a certain shape.
