@@ -39,7 +39,6 @@ from sklearn.metrics.pairwise import cosine_similarity
 from sklearn.preprocessing import MinMaxScaler
 
 from nemo.utils import logging
-from nemo.utils.decorators.experimental import experimental
 
 scaler = MinMaxScaler(feature_range=(0, 1))
 
@@ -138,7 +137,6 @@ def getRepeatedList(mapping_argmat, score_mat_size):
     return repeat_list
 
 
-@experimental
 def get_argmin_mat(uniq_scale_dict):
     """
     Calculate the mapping between the base scale and other scales. A segment from a longer scale is
@@ -171,7 +169,6 @@ def get_argmin_mat(uniq_scale_dict):
     return session_scale_mapping_dict
 
 
-@experimental
 def getMultiScaleCosAffinityMatrix(uniq_embs_and_timestamps):
     """
     Calculate cosine similarity values among speaker embeddings for each scale then
