@@ -32,12 +32,10 @@ from nemo.collections.nlp.data.text_normalization import constants
 from nemo.collections.nlp.data.text_normalization.utils import read_data_file
 from nemo.core.classes import Dataset
 from nemo.utils import logging
-from nemo.utils.decorators.experimental import experimental
 
 __all__ = ['TextNormalizationDecoderDataset']
 
 
-@experimental
 class TextNormalizationDecoderDataset(Dataset):
     """
     Creates dataset to use to train a DuplexDecoderModel.
