@@ -12,5 +12,5 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from nemo.collections.nlp.metrics.classification_report import ClassificationReport
+from nemo.collections.nlp.metrics.classification_report import ClassificationReport, MultiLabelClassificationReport
 from nemo.collections.nlp.metrics.sequence_perplexity import SequencePerplexity
