@@ -36,7 +36,6 @@ from nemo.collections.nlp.data.dialogue_state_tracking_generative.sgd.prediction
 from nemo.collections.nlp.losses import SGDDialogueStateLoss
 from nemo.collections.nlp.models.nlp_model import NLPModel
 from nemo.collections.nlp.modules import SGDDecoder, SGDEncoder
-from nemo.collections.nlp.modules.common.lm_utils import get_lm_model
 from nemo.collections.nlp.parts.utils_funcs import tensor2list
 from nemo.core.classes.common import PretrainedModelInfo, typecheck
 from nemo.core.neural_types import NeuralType
@@ -52,35 +51,24 @@ class SGDQAModel(NLPModel):
     """Dialogue State Tracking Model SGD-QA"""
 
     @property
-    def input_types(self) -> Optional[Dict[str, NeuralType]]:
-        return self.bert_model.input_types
-
-    @property
-    def output_types(self) -> Optional[Dict[str, NeuralType]]:
-        return self.decoder.output_types
+    def output_module(self):
+        return self.decoder
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
-
         self.data_prepared = False
-        self.setup_tokenizer(cfg.tokenizer)
         super().__init__(cfg=cfg, trainer=trainer)
-        self.bert_model = get_lm_model(
-            pretrained_model_name=cfg.language_model.pretrained_model_name,
-            config_file=self.register_artifact('language_model.config_file', cfg.language_model.config_file),
-            config_dict=OmegaConf.to_container(cfg.language_model.config) if cfg.language_model.config else None,
-            checkpoint_file=cfg.language_model.lm_checkpoint,
-            vocab_file=self.register_artifact('tokenizer.vocab_file', cfg.tokenizer.vocab_file),
-        )
-
         self.encoder = SGDEncoder(hidden_size=self.bert_model.config.hidden_size, dropout=self._cfg.encoder.dropout)
         self.decoder = SGDDecoder(embedding_dim=self.bert_model.config.hidden_size)
         self.loss = SGDDialogueStateLoss(reduction="mean")
 
     @typecheck()
-    def forward(self, input_ids, token_type_ids, attention_mask):
+    def forward(self, input_ids, attention_mask, token_type_ids):
         token_embeddings = self.bert_model(
             input_ids=input_ids, token_type_ids=token_type_ids, attention_mask=attention_mask
         )
+        if isinstance(token_embeddings, tuple):
+            token_embeddings = token_embeddings[0]
+
         encoded_utterance, token_embeddings = self.encoder(hidden_states=token_embeddings)
         (
             logit_intent_status,
@@ -238,7 +226,7 @@ class SGDQAModel(NLPModel):
         all_start_char_idx = []
         all_end_char_idx = []
 
-        if self.trainer.gpus and self.trainer.world_size > 1:
+        if self.trainer.devices and self.trainer.world_size > 1:
             world_size = self.trainer.world_size
             for ind in range(world_size):
                 all_example_id_num.append(torch.empty_like(example_id_num))
