@@ -22,6 +22,7 @@ from omegaconf.dictconfig import DictConfig
 from omegaconf.omegaconf import open_dict
 from pytorch_lightning.plugins.precision.native_amp import NativeMixedPrecisionPlugin
 from pytorch_lightning.trainer.trainer import Trainer
+from torch.utils.data import DataLoader, Dataset
 
 from nemo.collections.nlp.data.language_modeling.megatron.gpt_dataset import build_train_valid_test_datasets
 from nemo.collections.nlp.data.language_modeling.megatron.gpt_prompt_tuning_dataset import GPTPromptTuningDataset
@@ -37,21 +38,25 @@ from nemo.collections.nlp.modules.common.megatron.module import Float16Module
 from nemo.collections.nlp.modules.common.megatron.utils import (
     average_losses_across_data_parallel_group,
     get_ltor_masks_and_position_ids,
+    get_params_for_weight_decay_optimization,
 )
+from nemo.collections.nlp.modules.common.text_generation_utils import generate, get_computeprob_response
 from nemo.collections.nlp.modules.common.tokenizer_utils import get_nmt_tokenizer
+from nemo.collections.nlp.modules.common.transformer.text_generation import (
+    LengthParam,
+    OutputType,
+    SamplingParam,
+    TextGeneration,
+)
 from nemo.collections.nlp.parts.nlp_overrides import GradScaler
 from nemo.collections.nlp.parts.utils_funcs import get_last_rank
+from nemo.core.classes.common import PretrainedModelInfo
 from nemo.core.optim import MainParamsOptimizerWrapper, prepare_lr_scheduler
 from nemo.utils import AppState, logging
 
 try:
     from apex.transformer import parallel_state, tensor_parallel
-    from apex.contrib.layer_norm.layer_norm import FastLayerNorm
-    from apex.normalization.fused_layer_norm import FusedLayerNorm  # NOQA
-    from apex.transformer.pipeline_parallel.schedules.common import (
-        build_model,
-        listify_model,
-    )
+    from apex.transformer.pipeline_parallel.schedules.common import build_model
     from apex.transformer.pipeline_parallel.schedules.fwd_bwd_pipelining_without_interleaving import (
         forward_backward_pipelining_without_interleaving,
     )
@@ -63,34 +68,7 @@ except (ImportError, ModuleNotFoundError):
     HAVE_APEX = False
 
 
-def _get_params_for_weight_decay_optimization(
-    model: Union[torch.nn.Module, List[torch.nn.Module]],
-) -> Dict[str, torch.nn.Parameter]:
-    """Divide params into with-weight-decay and without-weight-decay groups.
-
-    Layernorms and biases will have no weight decay but the rest will.
-    """
-    modules = listify_model(model)
-    weight_decay_params = {'params': []}
-    no_weight_decay_params = {'params': [], 'weight_decay': 0.0}
-    for module in modules:
-        for module_ in module.modules():
-            if isinstance(module_, (FusedLayerNorm, FastLayerNorm)):
-                no_weight_decay_params['params'].extend(
-                    [p for p in list(module_._parameters.values()) if p is not None]
-                )
-            else:
-                weight_decay_params['params'].extend(
-                    [p for n, p in list(module_._parameters.items()) if p is not None and n != 'bias']
-                )
-                no_weight_decay_params['params'].extend(
-                    [p for n, p in list(module_._parameters.items()) if p is not None and n == 'bias']
-                )
-
-    return weight_decay_params, no_weight_decay_params
-
-
-class MegatronGPTModel(NLPModel):
+class MegatronGPTModel(NLPModel, TextGeneration):
     """
     Megatron GPT pretraining and prompt tuning
     """
@@ -100,13 +78,14 @@ class MegatronGPTModel(NLPModel):
             raise ImportError(
                 "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
             )
-        super().__init__(cfg, trainer=trainer)
-        self.cfg = cfg
+        # this prevents base constructor from initializing tokenizer
+        self.tokenizer = None
+        super().__init__(cfg, trainer=trainer, no_lm_init=True)
 
         self._validate_trainer()
 
         # used in NVIDIA NGC PyTorch containers
-        # self._enable_nvidia_optimizations()
+        self._enable_nvidia_optimizations()
 
         if self.cfg.get('use_cpu_initialization', False) is False:
             torch.cuda.set_device(trainer.local_rank)
@@ -126,9 +105,10 @@ class MegatronGPTModel(NLPModel):
         self.tokenizer = get_nmt_tokenizer(
             library=self.cfg.tokenizer.library,
             model_name=self.cfg.tokenizer.type,
-            tokenizer_model=self.register_artifact("tokenizer_model", self.cfg.tokenizer.model),
-            vocab_file=self.register_artifact("vocab_file", self.cfg.tokenizer.vocab_file),
-            merges_file=self.register_artifact("merges_file", self.cfg.tokenizer.merge_file),
+            tokenizer_model=self.register_artifact("tokenizer.model", self.cfg.tokenizer.model),
+            vocab_file=self.register_artifact("tokenizer.vocab_file", self.cfg.tokenizer.vocab_file),
+            merges_file=self.register_artifact("tokenizer.merge_file", self.cfg.tokenizer.merge_file),
+            delimiter=self.cfg.tokenizer.get('delimiter', None),
         )
 
         vocab_size = self.tokenizer.vocab_size
@@ -194,6 +174,15 @@ class MegatronGPTModel(NLPModel):
         else:
             raise ValueError('precision must be in [32, 16, "bf16"]')
 
+        # configuration used for inference
+        self._inference_config = None
+
+    def set_inference_config(self, inference_config):
+        self._inference_config = inference_config
+
+    def get_inference_config(self):
+        return self._inference_config
+
     def model_provider_func(self, pre_process, post_process):
         """Model depends on pipeline paralellism."""
         model = GPTModel(
@@ -233,7 +222,7 @@ class MegatronGPTModel(NLPModel):
 
     def setup_optimizer_param_groups(self):
         """ModelPT override. Optimizer will get self._optimizer_param_groups"""
-        self._optimizer_param_groups = _get_params_for_weight_decay_optimization([self.model])
+        self._optimizer_param_groups = get_params_for_weight_decay_optimization([self.model])
 
     def training_step(self, batch, batch_idx):
         """
@@ -442,15 +431,49 @@ class MegatronGPTModel(NLPModel):
 
     def get_forward_output_only_func(self):
         def fwd_output_only_func(batch, model):
-            batch = [x.cuda() for x in batch]
-
+            # batch = [x.cuda() for x in batch]
+            extra_arg = {}
             if self.use_soft_prompts:
-                tokens, attention_mask, position_ids, prompt_ids = batch
-                output_tensor = model(tokens, position_ids, attention_mask, prompt_ids=prompt_ids)
+                if len(batch) == 4:
+                    batch = [x.cuda() for x in batch]
+                    tokens, attention_mask, position_ids, prompt_ids = batch
+                    extra_arg['prompt_ids'] = prompt_ids
+                else:
+                    (
+                        tokens,
+                        attention_mask,
+                        position_ids,
+                        prompt_ids,
+                        set_inference_key_value_memory,
+                        inference_max_sequence_len,
+                    ) = batch
+                    tokens = tokens.cuda()
+                    attention_mask = attention_mask.cuda()
+                    position_ids = position_ids.cuda()
+                    extra_arg['prompt_ids'] = prompt_ids.cuda()
+                    extra_arg['set_inference_key_value_memory'] = set_inference_key_value_memory[0].item()
+                    extra_arg['inference_max_sequence_len'] = inference_max_sequence_len[0].item()
+                output_tensor = model(tokens, position_ids, attention_mask, **extra_arg)
             else:
-                tokens, attention_mask, position_ids = batch
-                attention_mask = attention_mask[0:1]
-                output_tensor = model(tokens, position_ids, attention_mask)
+                if len(batch) == 3:
+                    batch = [x.cuda() for x in batch]
+                    tokens, attention_mask, position_ids = batch
+                    attention_mask = attention_mask[0:1]
+                else:
+                    (
+                        tokens,
+                        attention_mask,
+                        position_ids,
+                        set_inference_key_value_memory,
+                        inference_max_sequence_len,
+                    ) = batch
+                    tokens = tokens.cuda()
+                    attention_mask = attention_mask.cuda()
+                    position_ids = position_ids.cuda()
+                    attention_mask = attention_mask[0:1]
+                    extra_arg['set_inference_key_value_memory'] = set_inference_key_value_memory[0].item()
+                    extra_arg['inference_max_sequence_len'] = inference_max_sequence_len[0].item()
+                output_tensor = model(tokens, position_ids, attention_mask, **extra_arg)
 
             def id_func(output_tensor):
                 return output_tensor, {'logits': output_tensor}
@@ -511,6 +534,8 @@ class MegatronGPTModel(NLPModel):
         return loss_mean
 
     def validation_epoch_end(self, outputs):
+        if not outputs:
+            return
         if parallel_state.is_pipeline_last_stage():
             # only the last pipeline parallel stages return loss
             averaged_loss = torch.stack(outputs).mean()
@@ -558,7 +583,6 @@ class MegatronGPTModel(NLPModel):
             return
 
         logging.info('Building GPT datasets.')
-        global_batch_size = self.trainer.world_size * self.cfg.micro_batch_size / self.cfg.tensor_model_parallel_size
         global_batch_size = self.cfg.global_batch_size
         max_train_steps = self.trainer.max_steps
         eval_iters = (max_train_steps // self.trainer.val_check_interval + 1) * self.trainer.limit_val_batches
@@ -662,7 +686,7 @@ class MegatronGPTModel(NLPModel):
         Args:
             stage (str, optional): Can be 'fit', 'validate', 'test' or 'predict'. Defaults to None.
         """
-        resume_checkpoint_path = self.trainer.checkpoint_connector.resume_from_checkpoint_fit_path
+        resume_checkpoint_path = self.trainer._checkpoint_connector.resume_from_checkpoint_fit_path
         if resume_checkpoint_path:
             try:
                 init_consumed_samples = int(
@@ -830,45 +854,6 @@ class MegatronGPTModel(NLPModel):
 
         self._optimizer_param_groups = weight_decay_params, no_weight_decay_params
 
-    @classmethod
-    def _bucketize_gpt_inference(cls, batch, use_soft_prompts=False):
-        batch_tokens, lens, tokens_to_generate, compute_logprobs = batch[:4]
-        batch_size = len(batch_tokens)
-        tokens_to_generate = tokens_to_generate[0]
-        batch_tokens = batch_tokens.tolist()
-
-        if use_soft_prompts:
-            prompt_tags = batch[4]
-
-        # unpad tokens
-        indxs = [index for index in range(batch_size)]
-        for lenn, index in zip(lens, indxs):
-            batch_tokens[index] = batch_tokens[index][:lenn]
-
-        # chunk tokens by same length
-        pre_buckets, lens = [], list(set(lens.tolist()))
-        for lenn in lens:
-            pre_buckets.append([(tokens, index) for index, tokens in enumerate(batch_tokens) if len(tokens) == lenn])
-
-        buckets, positions, bucket_prompt_tags = [], [], []
-
-        # get buckets and prompts initial positions
-        for bucket in pre_buckets:
-            buckets.append(torch.tensor([item[0] for item in bucket]).to(device='cuda'))
-            positions.append([item[1] for item in bucket])
-
-            # bucket prompt tags identically to their corresponding examples
-            if use_soft_prompts:
-                bucket_prompt_tags.append([prompt_tags[item[1]] for item in bucket])
-
-        # Flatten position list
-        positions = [item for sublist in positions for item in sublist]
-
-        # Form request
-        request = {"tokens": buckets, "prompt_tags": bucket_prompt_tags}
-
-        return request, positions, tokens_to_generate, compute_logprobs[0]
-
     def get_parameters(self):
         params = []
         for param_group in self._optimizer_param_groups:
@@ -876,147 +861,230 @@ class MegatronGPTModel(NLPModel):
                 params.append(param)
         return params
 
-    def predict_step(self, batch: Any, batch_idx: int, dataloader_idx: Optional[int] = None) -> Any:
-        request, positions, tokens_to_generate, compute_logprobs = MegatronGPTModel._bucketize_gpt_inference(
-            batch, self.use_soft_prompts
-        )
+    def generate(
+        self,
+        inputs: Union[List[str], torch.Tensor, List[dict]],
+        length_params: LengthParam,
+        sampling_params: SamplingParam = None,
+    ) -> OutputType:
 
-        if compute_logprobs:
-            response = self.compute_logprobs(request, positions)
-        else:
-            response = self.complete(request, positions, tokens_to_generate)
+        # check whether the DDP is initialized
+        if parallel_state.is_unitialized():
 
-        return response
+            class RequestDataSet(Dataset):
+                def __init__(self, sentences):
+                    super().__init__()
+                    self.sentences = sentences
 
-    def complete(self, request: Dict, positions: List, tokens_to_generate: int):
-        """
-            Autoregressively invokes language model in the inference mode
-        Args:
-            request:
-                * tokens: List of "buckets" with unpadded tokens of the same length
-                * prompt_tags: List of "buckets" where each bucket contains the prompt_tag strings
-                               specifying the prompt tag to use (optional)
-            positions: List with initial prompts positions
-            tokens_to_generate: int value denoting amount of tokens model should generate
+                def __len__(self):
+                    return len(self.sentences)
 
-        Returns:
-            response: A python list of tuples
-                (text, tokens, log_probs, offsets)
-                * text: string, inputted prompt + generated text by model
-                * tokens: list of tokens correspond to text
-                * log_probs: list of tokens log probabilities
-                * offsets: list of tokens start positions in text
+                def __getitem__(self, idx):
+                    return self.sentences[idx]
 
-        """
-        app_state = AppState()
+            # TODO, this is a little hacky. need to handle this nicely in the future
+            # run empty predict to initialize the DDP
+            ds = RequestDataSet([""])
+            request_dl = DataLoader(dataset=ds, batch_size=1)
+            self.trainer.predict(self, request_dl)
 
-        results = []
-        request_tokens = request["tokens"]
+        # set the default sampling params if it is None.
+        # default do greedy sampling
+        if sampling_params is None:
+            sampling_params: SamplingParam = {
+                "use_greedy": True,
+                "temperature": 1.0,
+                "top_k": 0,
+                "top_p": 1.0,
+                "repetition_penalty": 1.0,
+                "add_BOS": True,
+                "all_probs": False,
+                "compute_logprob": False,
+            }
 
-        for idx, tokens in enumerate(request_tokens):
-            micro_batch_size = tokens.shape[0]
-            _reconfigure_microbatch_calculator(
-                rank=app_state.global_rank,
-                rampup_batch_size=None,
-                global_batch_size=micro_batch_size,
-                micro_batch_size=micro_batch_size,
-                data_parallel_size=1,
+        # set the default length params if it is None.
+        # default do greedy sampling
+        if length_params is None:
+            length_params: LengthParam = {"min_length": 0, "max_length": 30}
+
+        # reproduce the old compute_prob method
+        # a very special case
+        if sampling_params['compute_logprob']:
+            # need to overwrite some configuration, make it immutable
+            sampling_params = sampling_params.copy()
+            length_params = length_params.copy()
+            length_params['max_length'] = 1
+            sampling_params['all_probs'] = True
+            sampling_params["add_BOS"] = False
+            sampling_params['use_greedy'] = True
+            response = generate(
+                self.cuda(),
+                inputs=inputs,
+                tokens_to_generate=length_params['max_length'],
+                all_probs=sampling_params['all_probs'],
+                temperature=sampling_params['temperature'],
+                add_BOS=sampling_params['add_BOS'],
+                top_k=sampling_params['top_k'],
+                top_p=sampling_params['top_p'],
+                greedy=sampling_params['use_greedy'],
+                repetition_penalty=sampling_params['repetition_penalty'],
+                min_tokens_to_generate=length_params['min_length'],
             )
+            compute_prob_response = get_computeprob_response(self.tokenizer, response, inputs)
+            return compute_prob_response
 
-            # For prompt tuned GPT models
-            if self.use_soft_prompts:
-                if self.cfg.get('pipeline_model_parallel_size', 1) > 1:
-                    raise ValueError('complete method is not yet supported for pipeline with soft prompts')
-                prompt_tags = request["prompt_tags"][idx]
-                prompt_tags_to_ids = dict(self.prompt_table)
-                prompt_ids = torch.tensor([prompt_tags_to_ids[tag] for tag in prompt_tags])
+        if isinstance(inputs, (list, tuple)):
+            if isinstance(inputs[0], (str, torch.Tensor)):
+                output = generate(
+                    self.cuda(),
+                    inputs=inputs,
+                    tokens_to_generate=length_params['max_length'],
+                    all_probs=sampling_params['all_probs'],
+                    temperature=sampling_params['temperature'],
+                    add_BOS=sampling_params['add_BOS'],
+                    top_k=sampling_params['top_k'],
+                    top_p=sampling_params['top_p'],
+                    greedy=sampling_params['use_greedy'],
+                    repetition_penalty=sampling_params['repetition_penalty'],
+                    min_tokens_to_generate=length_params['min_length'],
+                )
+                return output
+            elif isinstance(inputs[0], dict):
+                raise NotImplementedError("json object not implemented")
             else:
-                prompt_ids = None
+                raise NotImplementedError("unknown type is not implemented")
+        else:
+            raise NotImplementedError("unknown type is not implemented")
 
-            logsoftmaxlayer = torch.nn.LogSoftmax(dim=-1)
+    def predict_step(self, batch: Any, batch_idx: int, dataloader_idx: Optional[int] = None) -> Any:
+        inference_config = self.get_inference_config()
+        if inference_config is None:
+            return None
+        else:
+            # need to overwrite some configuration, make it immutable
+            inference_config = inference_config.copy()
+            compute_logprob = inference_config['compute_logprob']
+            if compute_logprob:
+                del inference_config['compute_logprob']
+                inference_config['inputs'] = batch
+                inference_config['tokens_to_generate'] = 1
+                inference_config['all_probs'] = True
+                inference_config["add_BOS"] = False
+                inference_config['greedy'] = True
+                response = generate(self, **inference_config)
+                compute_prob_response = get_computeprob_response(self.tokenizer, response, batch)
+                return compute_prob_response
+            else:
+                del inference_config['compute_logprob']
+                inference_config['inputs'] = batch
+                return generate(self, **inference_config)
 
-            for i in range(tokens_to_generate + 1):
-                if self.use_soft_prompts:
-                    batch_size = len(tokens)
-                    full_length = len(tokens[0]) + self.num_prompt_tokens
+    def init_new_prompts(self):
+        for idx, tag in enumerate(self.cfg.new_prompt_tags):
+            init_method = self.cfg.new_prompt_init_methods[idx]
 
-                    # Get postion ids for text after soft prompt
-                    position_ids = torch.arange(
-                        start=self.num_prompt_tokens, end=full_length, dtype=torch.long, device=self.device
-                    )
-                    position_ids = position_ids.unsqueeze(0).expand_as(tokens).clone()
+            if init_method == "text":
+                init_text = self.cfg.new_prompt_init_text[idx]
+                self.init_prompt_from_text(tag, init_text)
 
-                    # Make attention mask starting with first token in soft prompt
-                    attention_mask = torch.tril(
-                        torch.ones((batch_size, full_length, full_length), device=self.device)
-                    ).view(batch_size, 1, full_length, full_length)
-                    attention_mask = attention_mask < 0.5
+            elif init_method == 'random':
+                self.init_prompt_from_random(tag)
 
-                else:
-                    attention_mask, _, position_ids = get_ltor_masks_and_position_ids(
-                        data=tokens,
-                        eod_token=self.tokenizer.eos_id,
-                        reset_position_ids=self.cfg.get('reset_position_ids', False),
-                        reset_attention_mask=self.cfg.get('reset_attention_mask', False),
-                        eod_mask_loss=self.cfg.get('eod_mask_loss', False),
-                    )
-                attention_mask_repeat = torch.concat([attention_mask for _ in range(micro_batch_size)])
-                if self.use_soft_prompts:
-                    batch = [tokens, attention_mask_repeat, position_ids, prompt_ids]
-                else:
-                    batch = [tokens, attention_mask_repeat, position_ids]
-                tensor_shape = [tokens.shape[1], micro_batch_size, self.cfg.hidden_size]
-                if self.cfg.get('pipeline_model_parallel_size', 1) > 1:
-                    output_tensor = forward_backward_pipelining_without_interleaving(
-                        forward_step_func=self.get_forward_output_only_func(),
-                        batch=batch,
-                        model=self.model,
-                        forward_only=True,
-                        tensor_shape=tensor_shape,
-                        dtype=self.autocast_dtype,
-                    )
-                else:
-                    output_tensor = forward_backward_no_pipelining(
-                        forward_step_func=self.get_forward_output_only_func(),
-                        batch=batch,
-                        model=self.model,
-                        forward_only=True,
-                        tensor_shape=tensor_shape,
-                        dtype=self.autocast_dtype,
-                    )
-
-                # get output tensor
-                if parallel_state.is_pipeline_last_stage():
-                    output_tensor = output_tensor[0]['logits']
-                    output_tensor = tensor_parallel.gather_from_tensor_model_parallel_region(output_tensor)
-
-                    log_probs, token_ids = torch.max(logsoftmaxlayer(output_tensor), dim=-1)
-                    tokens = torch.cat([tokens, torch.unsqueeze(token_ids[:, -1], 1)], dim=1)
-                else:
-                    log_probs = torch.zeros((tokens.shape[0], tokens.shape[1]), dtype=torch.float).cuda()
-                    tokens = torch.zeros((tokens.shape[0], tokens.shape[1] + 1), dtype=tokens.dtype).cuda()
-
-                torch.distributed.broadcast(tokens, get_last_rank())
-                torch.distributed.broadcast(log_probs, get_last_rank())
-
-            # add to results as (text, tokens, log_probs, offsets)
-            for token, prob in zip(tokens, log_probs.tolist()):
-                results.append(
-                    (self.tokenizer.ids_to_text(token[:-1]), self.tokenizer.ids_to_tokens(token[:-1]), prob, [0],)
+            else:
+                raise AttributeError(
+                    f'\n Soft prompt init method {init_method} is not recognized\
+                                        please use text or random'
                 )
 
-        # offsets calculation
-        for item in results:
-            for index, token in enumerate(item[1]):
-                if index != len(item[1]) - 1:
-                    item[3].append(len(token) + item[3][-1])
+    def init_prompt_from_random(self, prompt_tag):
+        prompt_id = self._get_next_prompt_id()
+        self.model._init_prompt_from_random(prompt_tag, prompt_id)
+        self._add_prompt_tag(prompt_tag, prompt_id)
 
-        # return prompts in the order that they were input
-        response = [0 for i in range(len(positions))]
-        for item, index in zip(results, positions):
-            response[index] = item
+    def init_prompt_from_text(self, prompt_tag, init_text):
+        prompt_id = self._get_next_prompt_id()
+        init_token_ids = self.tokenizer.text_to_ids(init_text)
+        self.model._init_prompt_from_text(prompt_tag, prompt_id, init_token_ids)
+        self._add_prompt_tag(prompt_tag, prompt_id)
 
-        return response
+    def get_prompt_table(self):
+        if hasattr(self, 'prompt_table'):
+            return self.prompt_table
+
+    def list_available_models(self):
+        return None
+
+    def _get_next_prompt_id(self):
+        self.next_prompt_id += 1
+        return self.next_prompt_id
+
+    def _add_prompt_tag(self, prompt_tag, prompt_id):
+        if not hasattr(self, 'prompt_table'):
+            raise AttributeError('Please set "use_soft_prompts" in cfg to True')
+
+        self.prompt_table.add((prompt_tag, prompt_id))
+        self.prompts_to_tune.add(prompt_tag)
+
+        # Add new prompt tag to cfg for loading prompt table at inference
+        with open_dict(self.cfg):
+            self.cfg.existing_prompt_tags = list(self.prompt_table)
+
+    def _vocab_size_with_padding(self, orig_vocab_size, make_vocab_size_divisible_by, tensor_model_parallel_size):
+        """Pad vocab size so it is divisible by model parallel size and
+        still having GPU friendly size."""
+
+        after = orig_vocab_size
+        multiple = make_vocab_size_divisible_by * tensor_model_parallel_size
+        while (after % multiple) != 0:
+            after += 1
+        logging.info(
+            f'Padded vocab_size: {after}, original vocab_size: {orig_vocab_size}, dummy tokens: {after - orig_vocab_size}.'
+        )
+        return after
+
+    def _enable_nvidia_optimizations(self):
+        "These optimizations are present in NVIDIA NGC PyTorch Containers"
+
+        # Version check
+        nvidia_torch_version = os.getenv('NVIDIA_PYTORCH_VERSION', None)
+        if nvidia_torch_version is not None:
+            NVIDIA_TORCH_MAJOR = int(nvidia_torch_version.split('.')[0])
+            NVIDIA_TORCH_MINOR = int(nvidia_torch_version.split('.')[1])
+
+            # Apex Persistent layer norm is supported from Nvidia PyTorch container v21.11
+            if NVIDIA_TORCH_MAJOR < 21 or (NVIDIA_TORCH_MAJOR == 21 and NVIDIA_TORCH_MINOR < 11):
+                self.cfg.persist_layer_norm = False
+
+            if NVIDIA_TORCH_MAJOR >= 21 or (NVIDIA_TORCH_MAJOR == 21 and NVIDIA_TORCH_MINOR >= 11):
+                # NVFUSER
+                torch._C._jit_set_profiling_executor(True)
+                torch._C._jit_set_profiling_mode(True)
+                torch._C._jit_override_can_fuse_on_cpu(False)
+                torch._C._jit_override_can_fuse_on_gpu(False)
+                torch._C._jit_set_texpr_fuser_enabled(False)
+                torch._C._jit_set_nvfuser_enabled(True)
+                torch._C._debug_set_autodiff_subgraph_inlining(False)
+
+        else:
+            # Not a Nvidia container. Dependency check is on users
+            pass
+
+    def transfer_batch_to_device(self, batch: Any, device: torch.device, dataloader_idx: int) -> Any:
+        """ PTL hook: https://pytorch-lightning.readthedocs.io/en/latest/common/lightning_module.html#transfer-batch-to-device
+            When using pipeline parallelism, we need the global batch to remain on the CPU,
+            since the memory overhead will be too high when using a large number of microbatches.
+            Microbatches are transferred from CPU to GPU inside the pipeline.
+        """
+        return batch
+
+    def _validate_trainer(self):
+        """ Certain trainer configurations can break training.
+            Here we try to catch them and raise an error.
+        """
+        if self.trainer.accumulate_grad_batches > 1:
+            raise ValueError(
+                f'Gradient accumulation is done within training_step. trainer.accumulate_grad_batches must equal 1'
+            )
 
     def compute_logprobs(self, request: Dict, positions: List):
         """
@@ -1142,109 +1210,19 @@ class MegatronGPTModel(NLPModel):
 
         return response
 
-    def init_new_prompts(self):
-        for idx, tag in enumerate(self.cfg.new_prompt_tags):
-            init_method = self.cfg.new_prompt_init_methods[idx]
-
-            if init_method == "text":
-                init_text = self.cfg.new_prompt_init_text[idx]
-                self.init_prompt_from_text(tag, init_text)
-
-            elif init_method == 'random':
-                self.init_prompt_from_random(tag)
-
-            else:
-                raise AttributeError(
-                    f'\n Soft prompt init method {init_method} is not recognized\
-                                        please use text or random'
-                )
-
-    def init_prompt_from_random(self, prompt_tag):
-        prompt_id = self._get_next_prompt_id()
-        self.model._init_prompt_from_random(prompt_tag, prompt_id)
-        self._add_prompt_tag(prompt_tag, prompt_id)
-
-    def init_prompt_from_text(self, prompt_tag, init_text):
-        prompt_id = self._get_next_prompt_id()
-        init_token_ids = self.tokenizer.text_to_ids(init_text)
-        self.model._init_prompt_from_text(prompt_tag, prompt_id, init_token_ids)
-        self._add_prompt_tag(prompt_tag, prompt_id)
-
-    def get_prompt_table(self):
-        if hasattr(self, 'prompt_table'):
-            return self.prompt_table
-
-    def list_available_models(self):
-        return None
-
-    def _get_next_prompt_id(self):
-        self.next_prompt_id += 1
-        return self.next_prompt_id
-
-    def _add_prompt_tag(self, prompt_tag, prompt_id):
-        if not hasattr(self, 'prompt_table'):
-            raise AttributeError('Please set "use_soft_prompts" in cfg to True')
-
-        self.prompt_table.add((prompt_tag, prompt_id))
-        self.prompts_to_tune.add(prompt_tag)
-
-        # Add new prompt tag to cfg for loading prompt table at inference
-        with open_dict(self.cfg):
-            self.cfg.existing_prompt_tags = list(self.prompt_table)
-
-    def _vocab_size_with_padding(self, orig_vocab_size, make_vocab_size_divisible_by, tensor_model_parallel_size):
-        """Pad vocab size so it is divisible by model parallel size and
-        still having GPU friendly size."""
-
-        after = orig_vocab_size
-        multiple = make_vocab_size_divisible_by * tensor_model_parallel_size
-        while (after % multiple) != 0:
-            after += 1
-        logging.info(
-            f'Padded vocab_size: {after}, original vocab_size: {orig_vocab_size}, dummy tokens: {after - orig_vocab_size}.'
-        )
-        return after
-
-    def _enable_nvidia_optimizations(self):
-        "These optimizations are present in NVIDIA NGC PyTorch Containers"
-
-        # Version check
-        nvidia_torch_version = os.getenv('NVIDIA_PYTORCH_VERSION', None)
-        if nvidia_torch_version is not None:
-            NVIDIA_TORCH_MAJOR = int(nvidia_torch_version.split('.')[0])
-            NVIDIA_TORCH_MINOR = int(nvidia_torch_version.split('.')[1])
-
-            # Apex Persistent layer norm is supported from Nvidia PyTorch container v21.11
-            if NVIDIA_TORCH_MAJOR < 21 or (NVIDIA_TORCH_MAJOR == 21 and NVIDIA_TORCH_MINOR < 11):
-                self.cfg.persist_layer_norm = False
-
-            if NVIDIA_TORCH_MAJOR >= 21 or (NVIDIA_TORCH_MAJOR == 21 and NVIDIA_TORCH_MINOR >= 11):
-                # NVFUSER
-                torch._C._jit_set_profiling_executor(True)
-                torch._C._jit_set_profiling_mode(True)
-                torch._C._jit_override_can_fuse_on_cpu(False)
-                torch._C._jit_override_can_fuse_on_gpu(False)
-                torch._C._jit_set_texpr_fuser_enabled(False)
-                torch._C._jit_set_nvfuser_enabled(True)
-                torch._C._debug_set_autodiff_subgraph_inlining(False)
-
-        else:
-            # Not a Nvidia container. Dependency check is on users
-            pass
-
-    def transfer_batch_to_device(self, batch: Any, device: torch.device, dataloader_idx: int) -> Any:
-        """ PTL hook: https://pytorch-lightning.readthedocs.io/en/latest/common/lightning_module.html#transfer-batch-to-device
-            When using pipeline parallelism, we need the global batch to remain on the CPU,
-            since the memory overhead will be too high when using a large number of microbatches.
-            Microbatches are transferred from CPU to GPU inside the pipeline.
+    @classmethod
+    def list_available_models(cls) -> Optional[PretrainedModelInfo]:
         """
-        return batch
-
-    def _validate_trainer(self):
-        """ Certain trainer configurations can break training.
-            Here we try to catch them and raise an error.
+        This method returns a list of pre-trained model which can be instantiated directly from NVIDIA's NGC cloud.
+        Returns:
+            List of available pre-trained models.
         """
-        if self.trainer.accumulate_grad_batches > 1:
-            raise ValueError(
-                f'Gradient accumulation is done within training_step. trainer.accumulate_grad_batches must equal 1'
+        result = []
+        result.append(
+            PretrainedModelInfo(
+                pretrained_model_name="megatron_gpt_345m",
+                location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/megatron_gpt_345m/versions/1/files/megatron_gpt_345m.nemo",
+                description="345M parameter GPT generative Megatron model.",
             )
+        )
+        return result
