@@ -47,7 +47,7 @@ class TransformerLMModel(ModelPT):
         # Get global rank and total number of GPU workers for IterableDataset partitioning, if applicable
         self.world_size = 1
         if trainer is not None:
-            self.world_size = trainer.num_nodes * trainer.num_gpus
+            self.world_size = trainer.num_nodes * trainer.num_devices
 
         cfg = model_utils.convert_model_config_to_dict_config(cfg)
         cfg = model_utils.maybe_update_config_version(cfg)
