@@ -29,6 +29,6 @@ class RobertaEncoder(RobertaModel, BertModule):
     """
 
     @typecheck()
-    def forward(self, input_ids, token_type_ids, attention_mask):
+    def forward(self, input_ids, attention_mask, token_type_ids):
         res = super().forward(input_ids=input_ids, attention_mask=attention_mask)[0]
         return res
