@@ -18,6 +18,11 @@ import math
 
 import torch
 import torch.nn.functional as F
+from apex.transformer import parallel_state, tensor_parallel
+from apex.transformer.enums import AttnMaskType, AttnType, LayerType, ModelType
+from apex.transformer.functional.fused_softmax import FusedScaleMaskSoftmax
+from apex.transformer.parallel_state import get_tensor_model_parallel_world_size
+from apex.transformer.utils import divide as safe_divide
 
 from nemo.collections.nlp.modules.common.megatron.fused_bias_dropout_add import (
     bias_dropout_add,
@@ -27,21 +32,8 @@ from nemo.collections.nlp.modules.common.megatron.fused_bias_dropout_add import
 from nemo.collections.nlp.modules.common.megatron.fused_bias_gelu import fused_bias_gelu
 from nemo.collections.nlp.modules.common.megatron.fused_layer_norm import get_layer_norm
 from nemo.collections.nlp.modules.common.megatron.module import MegatronModule
-from nemo.collections.nlp.modules.common.megatron.utils import ApexGuardDefaults, attention_mask_func, erf_gelu
-from nemo.utils import logging
+from nemo.collections.nlp.modules.common.megatron.utils import attention_mask_func, erf_gelu
 
-try:
-    from apex.transformer import parallel_state, tensor_parallel
-    from apex.transformer.enums import AttnMaskType, AttnType, LayerType
-    from apex.transformer.functional.fused_softmax import FusedScaleMaskSoftmax
-    from apex.transformer.utils import divide as safe_divide
-
-    HAVE_APEX = True
-except (ImportError, ModuleNotFoundError):
-    HAVE_APEX = False
-
-    # fake missing classes with None attributes
-    AttnMaskType = AttnType = LayerType = ApexGuardDefaults()
 
 """ We use the following notation throughout this file:
      h: hidden size
@@ -59,6 +51,23 @@ except (ImportError, ModuleNotFoundError):
 """
 
 
+class ColumnLinear(tensor_parallel.ColumnParallelLinear):
+    # redefine forward only for non-parallel inference
+    def forward(self, input_):
+        world_size = get_tensor_model_parallel_world_size()
+        if input_.requires_grad or world_size > 1:
+            return tensor_parallel.ColumnParallelLinear.forward(self, input_)
+
+        # Matrix multiply.
+        output = torch.matmul(input_, self.weight.t())
+        if not self.skip_bias_add:
+            output = output + self.bias
+
+        output_bias = self.bias if self.skip_bias_add else None
+
+        return output, output_bias
+
+
 class ParallelMLP(MegatronModule):
     """MLP.
 
@@ -82,11 +91,11 @@ class ParallelMLP(MegatronModule):
         super(ParallelMLP, self).__init__()
         self.activation = activation
 
-        if activation not in ['gelu', 'geglu']:
-            raise ValueError(f"Activation {activation} not supported. Only gelu and geglu are supported.")
+        if activation not in ['gelu', 'geglu', 'reglu', 'swiglu']:
+            raise ValueError(f"Activation {activation} not supported. Only gelu, geglu, reglu, swiglu are supported.")
 
         # Project to 4h.
-        self.dense_h_to_4h = tensor_parallel.ColumnParallelLinear(
+        self.dense_h_to_4h = ColumnLinear(
             hidden_size,
             ffn_hidden_size,  # NOTE: When using geglu, divide ffn dim by 2/3 to keep overall params the same.
             gather_output=False,
@@ -95,28 +104,47 @@ class ParallelMLP(MegatronModule):
             use_cpu_initialization=use_cpu_initialization,
         )
 
-        if activation == 'geglu':
-            # Separate linear layer for GEGLU activation.
+        if activation in ['geglu', 'reglu', 'swiglu']:
+            # Separate linear layer for *GLU activations.
             # Source: https://github.com/huggingface/transformers/blob/bee361c6f1f7704f8c688895f2f86f6e5ff84727/src/transformers/models/t5/modeling_t5.py#L292
-            self.dense_h_to_4h_2 = tensor_parallel.ColumnParallelLinear(
+            self.dense_h_to_4h_2 = ColumnLinear(
                 hidden_size,
-                ffn_hidden_size,  # NOTE: When using geglu, divide ffn dim by 2/3 to keep overall params the same.
+                ffn_hidden_size,  # NOTE: When using *glu, divide ffn dim by 2/3 to keep overall params the same.
                 gather_output=False,
                 init_method=init_method,
                 skip_bias_add=True,
                 use_cpu_initialization=use_cpu_initialization,
             )
+            glu_activation_family = True
+        else:
+            glu_activation_family = False
+
+        if glu_activation_family and bias_gelu_fusion:
+            raise ValueError(
+                f"Cannot use bias_gelu_fusion with {activation} activation. Please turn bias gelu fusion off."
+            )
+
+        if glu_activation_family and openai_gelu:
+            raise ValueError(
+                f"Cannot use openai_gelu with specificed activation function : {activation} Please turn openai gelu off."
+            )
+
+        if glu_activation_family and onnx_safe:
+            raise ValueError(
+                f"Cannot use onnx_safe with specificed activation function : {activation} Please turn onnx safe off."
+            )
 
         self.bias_gelu_fusion = bias_gelu_fusion
-        self.activation_func = F.gelu
-        if activation == 'geglu':
-            self.activation_func = 'geglu'  # Implemented using F.gelu
-            if bias_gelu_fusion:
-                logging.warning("Bias Gelu Fusion is not supported for GEGLU activation. Running with pytorch F.gelu")
-        if openai_gelu:
+
+        if activation == "gelu":
+            self.activation_func = F.gelu
+        elif openai_gelu:
             self.activation_func = openai_gelu
         elif onnx_safe:
             self.activation_func = erf_gelu
+        else:
+            # Remaining acitvations are implemeted in the forward function.
+            self.activation_func = None
 
         # Project back to h.
         self.dense_4h_to_h = tensor_parallel.RowParallelLinear(
@@ -133,13 +161,22 @@ class ParallelMLP(MegatronModule):
         # [s, b, 4hp]
         intermediate_parallel, bias_parallel = self.dense_h_to_4h(hidden_states)
 
-        if self.activation == 'geglu':
+        if self.activation in ['geglu', 'reglu', 'swiglu']:
             intermediate_parallel_2, bias_parallel_2 = self.dense_h_to_4h_2(hidden_states)
 
         if self.activation == 'geglu':
             intermediate_parallel = F.gelu(intermediate_parallel + bias_parallel) * (
                 intermediate_parallel_2 + bias_parallel_2
             )
+        elif self.activation == 'swiglu':
+            # SiLU or sigmoid linear unit is the same as swish with beta = 1 (which is what https://arxiv.org/pdf/2002.05202.pdf uses.)
+            intermediate_parallel = F.silu(intermediate_parallel + bias_parallel) * (
+                intermediate_parallel_2 + bias_parallel_2
+            )
+        elif self.activation == 'reglu':
+            intermediate_parallel = F.relu(intermediate_parallel + bias_parallel) * (
+                intermediate_parallel_2 + bias_parallel_2
+            )
         elif self.bias_gelu_fusion and self.activation == 'gelu':
             intermediate_parallel = fused_bias_gelu(intermediate_parallel, bias_parallel)
         else:
@@ -172,6 +209,7 @@ class ParallelAttention(MegatronModule):
         use_cpu_initialization=False,
         masked_softmax_fusion=True,
         attention_dropout=0.1,
+        megatron_legacy=False,
     ):
         super(ParallelAttention, self).__init__()
 
@@ -182,6 +220,7 @@ class ParallelAttention(MegatronModule):
         self.layer_number = max(1, layer_number)
         self.attention_type = attention_type
         self.attn_mask_type = attn_mask_type
+        self.megatron_legacy = megatron_legacy
 
         if kv_channels is None:
             assert (
@@ -198,7 +237,7 @@ class ParallelAttention(MegatronModule):
 
         # Strided linear layer.
         if attention_type == AttnType.self_attn:
-            self.query_key_value = tensor_parallel.ColumnParallelLinear(
+            self.query_key_value = ColumnLinear(
                 hidden_size,
                 3 * projection_size,
                 gather_output=False,
@@ -207,11 +246,9 @@ class ParallelAttention(MegatronModule):
             )
         else:
             assert attention_type == AttnType.cross_attn
-            self.query = tensor_parallel.ColumnParallelLinear(
-                hidden_size, projection_size, gather_output=False, init_method=init_method
-            )
+            self.query = ColumnLinear(hidden_size, projection_size, gather_output=False, init_method=init_method)
 
-            self.key_value = tensor_parallel.ColumnParallelLinear(
+            self.key_value = ColumnLinear(
                 hidden_size, 2 * projection_size, gather_output=False, init_method=init_method
             )
 
@@ -248,9 +285,89 @@ class ParallelAttention(MegatronModule):
             use_cpu_initialization=use_cpu_initialization,
         )
 
-    def forward(self, hidden_states, attention_mask, layer_past=None, get_key_value=False, encoder_output=None):
+        # Inference key-value memory
+        self.inference_key_memory = None
+        self.inference_value_memory = None
+        self.inference_current_sequence_len = 0
+
+    def _allocate_memory(self, inference_max_sequence_len, batch_size, dtype):
+        return torch.empty(
+            inference_max_sequence_len,
+            batch_size,
+            self.num_attention_heads_per_partition,
+            self.hidden_size_per_attention_head,
+            dtype=dtype,
+            device=torch.cuda.current_device(),
+        )
+
+    def _transpose_last_dim(self, mixed_layer, num_splits, num_splits_first):
+        input_shape = mixed_layer.size()
+        if num_splits_first:
+            """[s, b, num_splits * np * hn]
+            -->(view) [s, b, num_splits, np, hn]
+            -->(tranpose) [s, b, np, num_splits, hn]
+            -->(view) [s, b, np * num_splits * hn] """
+
+            intermediate_shape = input_shape[:-1] + (
+                num_splits,
+                self.num_attention_heads_per_partition,
+                self.hidden_size_per_attention_head,
+            )
+
+            mixed_layer = mixed_layer.view(*intermediate_shape)
+            mixed_layer = mixed_layer.transpose(-2, -3).contiguous()
+        else:
+            """[s, b, np * hn * num_splits]
+            -->(view) [s, b, np, hn, num_splits]
+            -->(tranpose) [s, b, np, num_splits, hn]
+            -->(view) [s, b, np * num_splits * hn] """
+
+            intermediate_shape = input_shape[:-1] + (
+                self.num_attention_heads_per_partition,
+                self.hidden_size_per_attention_head,
+                num_splits,
+            )
+
+            mixed_layer = mixed_layer.view(*intermediate_shape)
+            mixed_layer = mixed_layer.transpose(-1, -2).contiguous()
+        mixed_layer = mixed_layer.view(*input_shape)
+
+        return mixed_layer
+
+    def forward(
+        self,
+        hidden_states,
+        attention_mask,
+        layer_past=None,
+        get_key_value=False,
+        encoder_output=None,
+        set_inference_key_value_memory=False,
+        inference_max_sequence_len=None,
+    ):
         # hidden_states: [sq, b, h]
 
+        # =================================================
+        # Pre-allocate memory for key-values for inference.
+        # =================================================
+        if set_inference_key_value_memory:
+            assert inference_max_sequence_len and inference_max_sequence_len > 0
+            self.inference_key_memory = self._allocate_memory(
+                inference_max_sequence_len, hidden_states.size(1), hidden_states.dtype
+            )
+            self.inference_value_memory = self._allocate_memory(
+                inference_max_sequence_len, hidden_states.size(1), hidden_states.dtype
+            )
+            self.inference_current_sequence_len = 0
+        # Some consistency check.
+        if inference_max_sequence_len:
+            assert self.inference_current_sequence_len < self.inference_key_memory.size(0)
+            assert inference_max_sequence_len == self.inference_key_memory.size(0)
+        # This is added for safety. In case inference_max_sequence_len
+        # is not provided, make sure there is no potential memory left
+        # from previous inference.
+        if not inference_max_sequence_len:
+            self.inference_key_memory = None
+            self.inference_value_memory = None
         # =====================
         # Query, Key, and Value
         # =====================
@@ -264,6 +381,8 @@ class ParallelAttention(MegatronModule):
                 self.num_attention_heads_per_partition,
                 3 * self.hidden_size_per_attention_head,
             )
+            if self.megatron_legacy:
+                mixed_x_layer = self._transpose_last_dim(mixed_x_layer, 3, True)
             mixed_x_layer = mixed_x_layer.view(*new_tensor_shape)
 
             # [sq, b, np, 3 * hn] --> 3 [sq, b, np, hn]
@@ -291,9 +410,22 @@ class ParallelAttention(MegatronModule):
             )
             query_layer = query_layer.view(*new_tensor_shape)
 
-        # ==================================
-        # Adjust key and value for inference
-        # ==================================
+        # ===================================================
+        # Adjust key, value, and attention mask for inference
+        # ===================================================
+
+        if inference_max_sequence_len:
+            # Adjust the range variables.
+            start = self.inference_current_sequence_len
+            self.inference_current_sequence_len += key_layer.size(0)
+            end = self.inference_current_sequence_len
+            # Copy key and values.
+            self.inference_key_memory[start:end, ...] = key_layer
+            self.inference_value_memory[start:end, ...] = value_layer
+            key_layer = self.inference_key_memory[:end, ...]
+            value_layer = self.inference_value_memory[:end, ...]
+            # Adjust attention mask
+            attention_mask = attention_mask[..., start:end, :end]
 
         if layer_past is not None:
             past_key, past_value = layer_past
@@ -441,6 +573,7 @@ class ParallelTransformerLayer_(MegatronModule):
         masked_softmax_fusion=True,
         attention_dropout=0.1,
         activation='gelu',
+        megatron_legacy=False,
     ):
         super(ParallelTransformerLayer_, self).__init__()
 
@@ -476,6 +609,7 @@ class ParallelTransformerLayer_(MegatronModule):
             use_cpu_initialization=use_cpu_initialization,
             masked_softmax_fusion=masked_softmax_fusion,
             attention_dropout=attention_dropout,
+            megatron_legacy=megatron_legacy,
         )
         self.hidden_dropout = hidden_dropout
         self.attention_dropout = attention_dropout
@@ -499,6 +633,7 @@ class ParallelTransformerLayer_(MegatronModule):
                 use_cpu_initialization=use_cpu_initialization,
                 masked_softmax_fusion=masked_softmax_fusion,
                 attention_dropout=attention_dropout,
+                megatron_legacy=megatron_legacy,
             )
             # Layernorm on the attention output.
             self.post_inter_attention_layernorm = get_layer_norm(hidden_size, layernorm_epsilon, persist_layer_norm)
@@ -524,6 +659,8 @@ class ParallelTransformerLayer_(MegatronModule):
         enc_dec_attn_mask=None,
         layer_past=None,
         get_key_value=False,
+        set_inference_key_value_memory=False,
+        inference_max_sequence_len=None,
     ):
         # hidden_states: [b, s, h]
 
@@ -531,7 +668,12 @@ class ParallelTransformerLayer_(MegatronModule):
         layernorm_output = self.input_layernorm(hidden_states)
         # Self attention.
         attention_output, attention_bias = self.self_attention(
-            layernorm_output, attention_mask, layer_past=layer_past, get_key_value=get_key_value
+            layernorm_output,
+            attention_mask,
+            layer_past=layer_past,
+            get_key_value=get_key_value,
+            set_inference_key_value_memory=set_inference_key_value_memory,
+            inference_max_sequence_len=inference_max_sequence_len,
         )
 
         if get_key_value:
@@ -555,11 +697,9 @@ class ParallelTransformerLayer_(MegatronModule):
         else:
             bias_dropout_add_func = get_bias_dropout_add(self.training)
 
-        # re-enable torch grad to enable fused optimization.
-        with torch.enable_grad():
-            layernorm_input = bias_dropout_add_func(
-                attention_output, attention_bias.expand_as(residual), residual, self.hidden_dropout
-            )
+        layernorm_input = bias_dropout_add_func(
+            attention_output, attention_bias.expand_as(residual), residual, self.hidden_dropout
+        )
 
         # Layer norm post the self attention.
         layernorm_output = self.post_attention_layernorm(layernorm_input)
@@ -574,11 +714,9 @@ class ParallelTransformerLayer_(MegatronModule):
             else:
                 residual = layernorm_input
 
-            # re-enable torch grad to enable fused optimization.
-            with torch.enable_grad():
-                layernorm_input = bias_dropout_add_func(
-                    attention_output, attention_bias.expand_as(residual), residual, self.hidden_dropout
-                )
+            layernorm_input = bias_dropout_add_func(
+                attention_output, attention_bias.expand_as(residual), residual, self.hidden_dropout
+            )
 
             # Layer norm post the decoder attention
             layernorm_output = self.post_inter_attention_layernorm(layernorm_input)
@@ -592,9 +730,7 @@ class ParallelTransformerLayer_(MegatronModule):
         else:
             residual = layernorm_input
 
-        # re-enable torch grad to enable fused optimization.
-        with torch.enable_grad():
-            output = bias_dropout_add_func(mlp_output, mlp_bias.expand_as(residual), residual, self.hidden_dropout)
+        output = bias_dropout_add_func(mlp_output, mlp_bias.expand_as(residual), residual, self.hidden_dropout)
 
         if get_key_value:
             output = [output, presents]
@@ -623,15 +759,30 @@ class ParallelTransformerLayer(ParallelTransformerLayer_):
         enc_dec_attn_mask=None,
         layer_past=None,
         get_key_value=False,
+        set_inference_key_value_memory=False,
+        inference_max_sequence_len=None,
     ):
-
         if self.dtype == torch.float32:
             return super().forward(
-                hidden_states, attention_mask, encoder_output, enc_dec_attn_mask, layer_past, get_key_value
+                hidden_states,
+                attention_mask,
+                encoder_output,
+                enc_dec_attn_mask,
+                layer_past,
+                get_key_value,
+                set_inference_key_value_memory,
+                inference_max_sequence_len,
             )
         with torch.autocast(device_type="cuda", dtype=self.dtype):
             return super().forward(
-                hidden_states, attention_mask, encoder_output, enc_dec_attn_mask, layer_past, get_key_value
+                hidden_states,
+                attention_mask,
+                encoder_output,
+                enc_dec_attn_mask,
+                layer_past,
+                get_key_value,
+                set_inference_key_value_memory,
+                inference_max_sequence_len,
             )
 
 
@@ -666,6 +817,8 @@ class ParallelTransformer(MegatronModule):
         openai_gelu=False,
         onnx_safe=False,
         activation='gelu',
+        model_type=ModelType.encoder_or_decoder,
+        megatron_legacy=False,
     ):
         super(ParallelTransformer, self).__init__()
 
@@ -680,17 +833,20 @@ class ParallelTransformer(MegatronModule):
         self.post_process = post_process
         self.input_tensor = None
         self.self_attn_mask_type = self_attn_mask_type
+        self.model_type = model_type
 
         # Store activation checkpointing flag.
         self.activations_checkpoint_method = activations_checkpoint_method
         self.activations_checkpoint_num_layers = activations_checkpoint_num_layers
 
-        # Number of layers.
-        assert (
-            num_layers % parallel_state.get_pipeline_model_parallel_world_size() == 0
-        ), 'num_layers must be divisible by pipeline_model_parallel_size'
-        self.num_layers = num_layers // parallel_state.get_pipeline_model_parallel_world_size()
+        if self.model_type == ModelType.encoder_or_decoder:
+            assert (
+                num_layers % parallel_state.get_pipeline_model_parallel_world_size() == 0
+            ), 'num_layers must be divisible by pipeline_model_parallel_size'
 
+        # TODO: Add similar assert for encoder-decoder.
+
+        self.num_layers = self.get_num_layers(num_layers)
         # Transformer layers.
         def build_layer(layer_number):
             return ParallelTransformerLayer(
@@ -716,12 +872,14 @@ class ParallelTransformer(MegatronModule):
                 openai_gelu=openai_gelu,
                 onnx_safe=onnx_safe,
                 activation=activation,
+                megatron_legacy=megatron_legacy,
             )
 
-        if parallel_state.get_virtual_pipeline_model_parallel_rank() is not None:
+        if parallel_state.get_virtual_pipeline_model_parallel_world_size() is not None:
             assert num_layers % parallel_state.get_virtual_pipeline_model_parallel_world_size() == 0, (
                 'num_layers_per_stage must be divisible by ' 'virtual_pipeline_model_parallel_size'
             )
+            assert self.model_type != ModelType.encoder_or_decoder
             # Number of layers in each model chunk is the number of layers in the stage,
             # divided by the number of model chunks in a stage.
             self.num_layers = self.num_layers // parallel_state.get_virtual_pipeline_model_parallel_world_size()
@@ -738,7 +896,18 @@ class ParallelTransformer(MegatronModule):
             ) + (parallel_state.get_pipeline_model_parallel_rank() * self.num_layers)
         else:
             # Each stage gets a contiguous set of layers.
-            offset = parallel_state.get_pipeline_model_parallel_rank() * self.num_layers
+            if (
+                self.model_type == ModelType.encoder_and_decoder
+                and parallel_state.get_pipeline_model_parallel_world_size() > 1
+            ):
+                pipeline_rank = parallel_state.get_pipeline_model_parallel_rank()
+                if layer_type == LayerType.encoder:
+                    offset = pipeline_rank * self.num_layers
+                else:
+                    num_ranks_in_enc = parallel_state.get_pipeline_model_parallel_split_rank()
+                    offset = (pipeline_rank - num_ranks_in_enc) * self.num_layers
+            else:
+                offset = parallel_state.get_pipeline_model_parallel_rank() * self.num_layers
 
         self.layers = torch.nn.ModuleList([build_layer(i + 1 + offset) for i in range(self.num_layers)])
 
@@ -749,6 +918,31 @@ class ParallelTransformer(MegatronModule):
     def _get_layer(self, layer_number):
         return self.layers[layer_number]
 
+    def get_num_layers(self, num_layers):
+        """Compute the number of transformer layers resident on the current rank."""
+        if parallel_state.get_pipeline_model_parallel_world_size() > 1:
+            if self.model_type == ModelType.encoder_and_decoder:
+                assert parallel_state.get_pipeline_model_parallel_split_rank() is not None
+                num_ranks_in_encoder = parallel_state.get_pipeline_model_parallel_split_rank()
+                num_ranks_in_decoder = parallel_state.get_pipeline_model_parallel_world_size() - num_ranks_in_encoder
+                assert (
+                    num_layers % num_ranks_in_encoder == 0
+                ), 'num_layers must be divisible by number of ranks given to encoder'
+                assert (
+                    num_layers % num_ranks_in_decoder == 0
+                ), 'num_layers must be divisible by number of ranks given to decoder'
+                if parallel_state.is_pipeline_stage_before_split():
+                    num_layers = num_layers // num_ranks_in_encoder
+                else:
+                    num_layers = num_layers // num_ranks_in_decoder
+            else:
+                assert (
+                    num_layers % parallel_state.get_pipeline_model_parallel_world_size() == 0
+                ), 'num_layers must be divisible by pipeline_model_parallel_size'
+                num_layers = num_layers // parallel_state.get_pipeline_model_parallel_world_size()
+
+        return num_layers
+
     def _checkpointed_forward(self, hidden_states, attention_mask, encoder_output, enc_dec_attn_mask):
         """Forward method with activation checkpointing."""
 
@@ -816,8 +1010,13 @@ class ParallelTransformer(MegatronModule):
         get_key_value=False,
         encoder_output=None,
         enc_dec_attn_mask=None,
+        set_inference_key_value_memory=False,
+        inference_max_sequence_len=None,
     ):
         # Checks.
+        if inference_max_sequence_len:
+            assert self.activations_checkpoint_method is None, 'inference does not work with activation checkpointing'
+
         if layer_past is not None:
             assert get_key_value, 'for not None values in layer_past, ' 'expected get_key_value to be set'
         if get_key_value:
@@ -859,6 +1058,8 @@ class ParallelTransformer(MegatronModule):
                     enc_dec_attn_mask=enc_dec_attn_mask,
                     layer_past=past,
                     get_key_value=get_key_value,
+                    set_inference_key_value_memory=set_inference_key_value_memory,
+                    inference_max_sequence_len=inference_max_sequence_len,
                 )
                 if get_key_value:
                     hidden_states, present = hidden_states
