@@ -47,7 +47,7 @@ class WaveglowConfig:
     validation_ds: Optional[Dict[Any, Any]] = None
 
 
-@deprecated(version="1.8", explanation="UniGlowModel will be removed. Use WaveGlowModel or HifiGanModel instead.")
+@deprecated(version="1.9", explanation="UniGlowModel will be removed. Use WaveGlowModel or HifiGanModel instead.")
 class UniGlowModel(GlowVocoder):
     """Waveglow model used to convert betweeen spectrograms and audio"""
 
@@ -264,3 +264,17 @@ class UniGlowModel(GlowVocoder):
         audio = split_view(audio, self._cfg.uniglow.n_group, 1).permute(0, 2, 1)
         upsample_factor = audio.shape[2] // spec.shape[2]
         return upsample_factor
+
+    def load_state_dict(self, state_dict, strict=True):
+        """Stopgap measure to keep old checkpoints working before full model deprecation.
+
+        This override fiddles with the state_dict in order to keep functionality from old checkpoints after the
+        switch from torch_stft. It will be removed when this model is deprecated.
+        """
+        if 'audio_to_melspec_precessor.stft.forward_basis' in state_dict:
+            logging.warning("Loading old checkpoint, defaulting to hann window.")
+            window_dim = state_dict['audio_to_melspec_precessor.stft.forward_basis'].shape[-1]
+            state_dict['audio_to_melspec_precessor.window'] = torch.hann_window(window_dim)
+            del state_dict['audio_to_melspec_precessor.stft.forward_basis']
+            del state_dict['audio_to_melspec_precessor.stft.inverse_basis']
+        super().load_state_dict(state_dict, strict=strict)
