@@ -32,9 +32,7 @@ from pytorch_lightning.callbacks import Callback, ModelCheckpoint
 from pytorch_lightning.callbacks.timer import Interval, Timer
 from pytorch_lightning.loggers import LoggerCollection as _LoggerCollection
 from pytorch_lightning.loggers import TensorBoardLogger, WandbLogger
-from pytorch_lightning.plugins.training_type.ddp import DDPPlugin
-from pytorch_lightning.trainer.states import RunningStage
-from pytorch_lightning.utilities.distributed import rank_zero_info
+from pytorch_lightning.strategies.ddp import DDPStrategy
 
 from nemo.constants import NEMO_ENV_VARNAME_TESTING, NEMO_ENV_VARNAME_VERSION
 from nemo.utils import logging, timers
@@ -76,7 +74,7 @@ class CallbackParams:
     save_top_k: Optional[int] = 3
     save_weights_only: Optional[bool] = False
     mode: Optional[str] = "min"
-    every_n_val_epochs: Optional[int] = 1
+    every_n_epochs: Optional[int] = 1
     prefix: Optional[str] = None  # If None, exp_manager will attempt to handle the filepath
     postfix: str = ".nemo"
     save_best_model: bool = False
@@ -139,10 +137,10 @@ class TimingCallback(Callback):
         self.timer.stop(name)
         pl_module.log(name, self.timer[name], on_step=True, on_epoch=False)
 
-    def on_train_batch_start(self, trainer, pl_module, batch, batch_idx, dataloader_idx):
+    def on_train_batch_start(self, trainer, pl_module, batch, batch_idx):
         self._on_batch_start("train_step_timing")
 
-    def on_train_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
+    def on_train_batch_end(self, trainer, pl_module, outputs, batch, batch_idx):
         self._on_batch_end("train_step_timing", pl_module)
 
     def on_validation_batch_start(self, trainer, pl_module, batch, batch_idx, dataloader_idx):
@@ -194,7 +192,7 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
                 lightning's TensorboardLogger system of using version_{int}.
             - use_datetime_version (bool): Whether to use a datetime string for version. Defaults to True.
             - resume_if_exists (bool): Whether this experiment is resuming from a previous run. If True, it sets
-                trainer.checkpoint_connector.resume_from_checkpoint_fit_path so that the trainer should auto-resume. exp_manager will move files
+                trainer._checkpoint_connector.resume_from_checkpoint_fit_path so that the trainer should auto-resume. exp_manager will move files
                 under log_dir to log_dir/run_{int}. Defaults to False. From v1.0.0, when resume_if_exists is True,
                 we would not create version folders to make it easier to find the log folder for next runs.
             - resume_past_end (bool): exp_manager errors out if resume_if_exists is True and a checkpoint matching
@@ -226,7 +224,7 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
     # Add rank information to logger
     # Note: trainer.global_rank and trainer.is_global_zero are not set until trainer.fit, so have to hack around it
     local_rank = int(os.environ.get("LOCAL_RANK", 0))
-    global_rank = trainer.node_rank * trainer.num_gpus + local_rank
+    global_rank = trainer.node_rank * trainer.num_devices + local_rank
     logging.rank = global_rank
     world_size = trainer.world_size
 
@@ -371,7 +369,7 @@ def error_checks(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictC
             "You are running multi-node training without SLURM handling the processes."
             " Please note that this is not tested in NeMo and could result in errors."
         )
-    if trainer.num_gpus > 1 and not isinstance(trainer.accelerator.training_type_plugin, DDPPlugin):
+    if trainer.num_devices > 1 and not isinstance(trainer.strategy, DDPStrategy):
         logging.error(
             "You are running multi-gpu without ddp.Please note that this is not tested in NeMo and could result in "
             "errors."
@@ -385,7 +383,7 @@ def check_resume(
     resume_ignore_no_checkpoint: bool = False,
 ):
     """Checks that resume=True was used correctly with the arguments pass to exp_manager. Sets
-    trainer.checkpoint_connector.resume_from_checkpoint_fit_path as necessary.
+    trainer._checkpoint_connector.resume_from_checkpoint_fit_path as necessary.
 
     Returns:
         log_dir (Path): the log_dir
@@ -442,7 +440,7 @@ def check_resume(
         logging.info(f"Resuming from {last_checkpoints[0]}")
         checkpoint = last_checkpoints[0]
 
-    trainer.checkpoint_connector.resume_from_checkpoint_fit_path = str(checkpoint)
+    trainer._checkpoint_connector.resume_from_checkpoint_fit_path = str(checkpoint)
 
     if is_global_rank_zero():
         # Check to see if any files exist that need to be moved
@@ -661,7 +659,7 @@ def configure_loggers(
     logger_list = (
         LoggerList(logger_list, nemo_name=name, nemo_version=version) if len(logger_list) > 1 else logger_list[0]
     )
-    trainer.logger_connector.configure_logger(logger_list)
+    trainer._logger_connector.configure_logger(logger_list)
 
 
 class NeMoModelCheckpoint(ModelCheckpoint):
@@ -799,13 +797,15 @@ class NeMoModelCheckpoint(ModelCheckpoint):
 
         # Load the best model and then re-save it
         if self.save_best_model:
+            # wait for all processes
+            trainer.training_type_plugin.barrier("SaveBestCheckpointConnector.resume_end")
             if self.best_model_path == "":
                 logging.warning(
                     f"{self} was told to save the best checkpoint at the end of training, but no saved checkpoints "
                     "were found. Saving latest model instead."
                 )
             else:
-                trainer.checkpoint_connector.restore(self.best_model_path)
+                trainer._checkpoint_connector.restore(self.best_model_path)
 
         if self.save_nemo_on_train_end:
             pl_module.save_to(save_path=os.path.join(self.dirpath, self.prefix + self.postfix))
@@ -885,7 +885,7 @@ def configure_checkpointing(
             )
 
     checkpoint_callback = NeMoModelCheckpoint(n_resume=resume, **params)
-    checkpoint_callback.last_model_path = trainer.checkpoint_connector.resume_from_checkpoint_fit_path or ""
+    checkpoint_callback.last_model_path = trainer._checkpoint_connector.resume_from_checkpoint_fit_path or ""
     if 'mp_rank' in checkpoint_callback.last_model_path or 'tp_rank' in checkpoint_callback.last_model_path:
         checkpoint_callback.last_model_path = uninject_model_parallel_rank(checkpoint_callback.last_model_path)
     trainer.callbacks.append(checkpoint_callback)
@@ -904,24 +904,9 @@ class StatelessTimer(Timer):
     def __init__(self, duration: timedelta = None, interval: str = Interval.step, verbose: bool = True,) -> None:
         super().__init__(duration, interval, verbose)
 
-    def on_save_checkpoint(self, trainer, pl_module, checkpoint) -> Dict[str, Any]:
-        return
+    # Override PTL Timer's state dict to not store elapsed time information so that we can restore and continue training.
+    def state_dict(self) -> Dict[str, Any]:
+        return {}
 
-    def on_load_checkpoint(self, trainer, pl_module, callback_state) -> None:
+    def load_state_dict(self, state_dict: Dict[str, Any]) -> None:
         return
-
-    def _check_time_remaining(self, trainer) -> None:
-        # Default timer only checks for train time exceeding max_time, this includes time for all stages.
-        train_duration = self.time_elapsed(RunningStage.TRAINING)
-        validation_duration = self.time_elapsed(RunningStage.VALIDATING)
-        test_duration = self.time_elapsed(RunningStage.TESTING)
-        total_duration = train_duration + validation_duration + test_duration
-        should_stop = total_duration >= self._duration
-        # should_stop = trainer.training_type_plugin.broadcast(should_stop)
-        should_stop = trainer.training_type_plugin.reduce_boolean_decision(should_stop)
-        trainer.should_stop = trainer.should_stop or should_stop
-        if should_stop and self._verbose:
-            rank_zero_info(f"Time limit reached. Signaling Trainer to stop.")
-            rank_zero_info(
-                f"Spent {timedelta(seconds=train_duration)} seconds on training, {timedelta(seconds=validation_duration)} seconds on validation and {timedelta(seconds=test_duration)} seconds on testing"
-            )
