@@ -44,7 +44,7 @@ class TestMoney:
     )
     normalizer_with_audio_en = (
         NormalizerWithAudio(input_case='cased', lang='en', cache_dir=CACHE_DIR, overwrite_cache=False)
-        if PYNINI_AVAILABLE
+        if PYNINI_AVAILABLE and CACHE_DIR
         else None
     )
 
@@ -57,5 +57,9 @@ class TestMoney:
     def test_norm(self, test_input, expected):
         pred = self.normalizer_en.normalize(test_input, verbose=False)
         assert pred == expected
-        pred_non_deterministic = self.normalizer_with_audio_en.normalize(test_input, n_tagged=100)
-        assert expected in pred_non_deterministic
+
+        if self.normalizer_with_audio_en:
+            pred_non_deterministic = self.normalizer_with_audio_en.normalize(
+                test_input, n_tagged=30, punct_post_process=False,
+            )
+            assert expected in pred_non_deterministic
